@@ -4,6 +4,7 @@ import json
 import logging
 import threading
 import uuid
+import warnings
 from concurrent.futures import Future
 from copy import copy
 from hashlib import md5
@@ -42,12 +43,12 @@ from crewai.utilities.config import process_config
 from crewai.utilities.constants import NOT_SPECIFIED, _NotSpecified
 from crewai.utilities.guardrail import process_guardrail, GuardrailResult
 from crewai.utilities.converter import Converter, convert_to_model
-from crewai.utilities.events import (
+from crewai.events.event_types import (
     TaskCompletedEvent,
     TaskFailedEvent,
     TaskStartedEvent,
 )
-from crewai.utilities.events.crewai_event_bus import crewai_event_bus
+from crewai.events.event_bus import crewai_event_bus
 from crewai.utilities.i18n import I18N
 from crewai.utilities.printer import Printer
 from crewai.utilities.string_utils import interpolate_only
@@ -72,6 +73,10 @@ class Task(BaseModel):
         output_pydantic: Pydantic model for task output.
         security_config: Security configuration including fingerprinting.
         tools: List of tools/resources limited for task execution.
+        allow_crewai_trigger_context: Optional flag to control crewai_trigger_payload injection.
+                              None (default): Auto-inject for first task only.
+                              True: Always inject trigger payload for this task.
+                              False: Never inject trigger payload, even for first task.
     """
 
     __hash__ = object.__hash__  # type: ignore
@@ -153,7 +158,11 @@ class Task(BaseModel):
         default=None,
         description="Function or string description of a guardrail to validate task output before proceeding to next task",
     )
-    max_retries: int = Field(
+    max_retries: Optional[int] = Field(
+        default=None,
+        description="[DEPRECATED] Maximum number of retries when guardrail fails. Use guardrail_max_retries instead. Will be removed in v1.0.0",
+    )
+    guardrail_max_retries: int = Field(
         default=3, description="Maximum number of retries when guardrail fails"
     )
     retry_count: int = Field(default=0, description="Current number of retries")
@@ -163,6 +172,10 @@ class Task(BaseModel):
     end_time: Optional[datetime.datetime] = Field(
         default=None, description="End time of the task execution"
     )
+    allow_crewai_trigger_context: Optional[bool] = Field(
+        default=None,
+        description="Whether this task should append 'Trigger Payload: {crewai_trigger_payload}' to the task description when crewai_trigger_payload exists in crew inputs.",
+    )
     model_config = {"arbitrary_types_allowed": True}
 
     @field_validator("guardrail")
@@ -346,6 +359,18 @@ class Task(BaseModel):
             )
         return self
 
+    @model_validator(mode="after")
+    def handle_max_retries_deprecation(self):
+        if self.max_retries is not None:
+            warnings.warn(
+                "The 'max_retries' parameter is deprecated and will be removed in CrewAI v1.0.0. "
+                "Please use 'guardrail_max_retries' instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            self.guardrail_max_retries = self.max_retries
+        return self
+
     def execute_sync(
         self,
         agent: Optional[BaseAgent] = None,
@@ -425,7 +450,7 @@ class Task(BaseModel):
 
             pydantic_output, json_output = self._export_output(result)
             task_output = TaskOutput(
-                name=self.name,
+                name=self.name or self.description,
                 description=self.description,
                 expected_output=self.expected_output,
                 raw=result,
@@ -442,9 +467,9 @@ class Task(BaseModel):
                     retry_count=self.retry_count,
                 )
                 if not guardrail_result.success:
-                    if self.retry_count >= self.max_retries:
+                    if self.retry_count >= self.guardrail_max_retries:
                         raise Exception(
-                            f"Task failed guardrail validation after {self.max_retries} retries. "
+                            f"Task failed guardrail validation after {self.guardrail_max_retries} retries. "
                             f"Last error: {guardrail_result.error}"
                         )
 
@@ -506,11 +531,11 @@ class Task(BaseModel):
     def _process_guardrail(self, task_output: TaskOutput) -> GuardrailResult:
         assert self._guardrail is not None
 
-        from crewai.utilities.events import (
+        from crewai.events.event_types import (
             LLMGuardrailCompletedEvent,
             LLMGuardrailStartedEvent,
         )
-        from crewai.utilities.events.crewai_event_bus import crewai_event_bus
+        from crewai.events.event_bus import crewai_event_bus
 
         crewai_event_bus.emit(
             self,
@@ -548,12 +573,23 @@ class Task(BaseModel):
             str: The formatted prompt string containing the task description,
                  expected output, and optional markdown formatting instructions.
         """
-        tasks_slices = [self.description]
+        description = self.description
+
+        should_inject = self.allow_crewai_trigger_context
+
+        if should_inject and self.agent:
+            crew = getattr(self.agent, "crew", None)
+            if crew and hasattr(crew, "_inputs") and crew._inputs:
+                trigger_payload = crew._inputs.get("crewai_trigger_payload")
+                if trigger_payload is not None:
+                    description += f"\n\nTrigger Payload: {trigger_payload}"
+
+        tasks_slices = [description]
 
         output = self.i18n.slice("expected_output").format(
             expected_output=self.expected_output
         )
-        tasks_slices = [self.description, output]
+        tasks_slices = [description, output]
 
         if self.markdown:
             markdown_instruction = """Your final answer MUST be formatted in Markdown syntax.
@@ -761,7 +797,9 @@ Follow these guidelines:
             if self.create_directory and not directory.exists():
                 directory.mkdir(parents=True, exist_ok=True)
             elif not self.create_directory and not directory.exists():
-                raise RuntimeError(f"Directory {directory} does not exist and create_directory is False")
+                raise RuntimeError(
+                    f"Directory {directory} does not exist and create_directory is False"
+                )
 
             with resolved_path.open("w", encoding="utf-8") as file:
                 if isinstance(result, dict):
