@@ -16,4 +16,4 @@ class _NotSpecified:
 # Unlike `None`, which might be a valid value from the user, `NOT_SPECIFIED` allows
 # us to distinguish between "not passed at all" and "explicitly passed None" or "[]".
 NOT_SPECIFIED = _NotSpecified()
-CREWAI_BASE_URL = "https://app.crewai.com/"
+CREWAI_BASE_URL = "https://app.crewai.com"
